local VERSION = "TradeLinks v3.1.2 - 9901";

--------------------------------------------------------------------------------
--                                                                            --
--------------------------------------------------------------------------------

local _G = _G;

-- Lua/bitlib Functions
local next = _G.next;
local max = _G.math.max;
local floor = _G.math.floor;
local strchar = _G.string.char;
local strbyte = _G.string.byte;
local bor = _G.bit.bor;
local blshift = _G.bit.lshift;
local band = _G.bit.band;

-- WoW API Functions
local CreateFrame = _G.CreateFrame;
local GetItemInfo = _G.GetItemInfo;
local GetSpellInfo = _G.GetSpellInfo;
local GetSpellLink = _G.GetSpellLink;
local IsModifiedClick = _G.IsModifiedClick;
local IsShiftKeyDown = _G.IsShiftKeyDown;
local UnitGUID = _G.UnitGUID;

-- WoW UI Objects
local ChatFontNormal = _G.ChatFontNormal;
local GameFontNormal = _G.GameFontNormal;

-- WoW UI Frames
local UIParent = _G.UIParent;
local GameTooltip = _G.GameTooltip;
local ChatFrame1 = _G.ChatFrame1;
local ChatFrameEditBox = _G.ChatFrameEditBox;

-- WoW UI Tables
local ShowUIPanel = _G.ShowUIPanel;
local SlashCmdList = _G.SlashCmdList;
local UIPanelWindows = _G.UIPanelWindows;

-- WoW UI Functions
local SetItemRef = _G.SetItemRef;
local UIDropDownMenu_AddButton = _G.UIDropDownMenu_AddButton;
local UIDropDownMenu_CreateInfo = _G.UIDropDownMenu_CreateInfo;
local UIDropDownMenu_Initialize = _G.UIDropDownMenu_Initialize;
local UIDropDownMenu_SetSelectedValue = _G.UIDropDownMenu_SetSelectedValue;
local UIDropDownMenu_SetWidth = _G.UIDropDownMenu_SetWidth;

--------------------------------------------------------------------------------
--                                                                            --
--------------------------------------------------------------------------------

local empty = {};
local LINE_COUNT = 23;

local TradeLinks = {};
_G.TradeLinks = TradeLinks;

local lib = LibStub("LibTradeLinks-1.0")

TradeLinks.Version = VERSION;
TradeLinks.Saved = { };
TradeLinks.Section = lib.SKILL_ALCHEMY;

--------------------------------------------------------------------------------
-- Return the table containing checked spellIds for the specified section     --
--------------------------------------------------------------------------------
function TradeLinks:GetSaved(section)
	if not self.Saved[section] then self.Saved[section] = {} end
	return self.Saved[section];
end

--------------------------------------------------------------------------------
-- Return the table containing spellIds for the specified section             --
--------------------------------------------------------------------------------
function TradeLinks:GetData(section)
	return lib:GetData(section) or empty;
end

--------------------------------------------------------------------------------
-- Reset the scrollbar to zero, and set min/max values to match the section   --
--------------------------------------------------------------------------------
function TradeLinks:UpdateSlider()
	local data = self:GetData(self.Section);
	self.Slider:SetMinMaxValues(1, max(1, #data - LINE_COUNT));
	self.Slider:SetValue(1);
end

--------------------------------------------------------------------------------
-- OnEnter function for the individual lines.                                 --
--------------------------------------------------------------------------------
function TradeLinks:OnEnter(frame)
	if IsShiftKeyDown() and frame.SpellId and frame.SpellId ~= 0 then
		local x, y = self.Frame:GetCenter();
		local x2, y2 = UIParent:GetCenter();
		
		if x < x2 then
			GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMRIGHT", 300, 20);
		else
			GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMLEFT", 0, 0);
		end
		
		GameTooltip:SetHyperlink("spell:" .. frame.SpellId);
	end
end

--------------------------------------------------------------------------------
-- OnLeave function for the individual lines.                                 --
--------------------------------------------------------------------------------
function TradeLinks:OnLeave(frame)
	GameTooltip:Hide();
end


--------------------------------------------------------------------------------
-- OnClick function for the individual lines.                                 --
--------------------------------------------------------------------------------
function TradeLinks:OnClick(button)
	if button.SpellId and button.SpellId ~= 0 then
		if not self.Saved[self.Section] then
			self.Saved[self.Section] = {};
		end
		self.Saved[self.Section][button.SpellId] = button:GetChecked();

		if IsModifiedClick("CHATLINK") and ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Insert(GetSpellLink(button.SpellId) or "");
		end
	end
end	

--------------------------------------------------------------------------------
-- UIDropDown SectionSelect function.                                         --
--------------------------------------------------------------------------------
function TradeLinks:SetSection(section)
	self.SectionSelect(nil, self, section);
end
function TradeLinks.SectionSelect(frame, self, section)
	self.Section = section;
	UIDropDownMenu_SetSelectedValue(self.DropDown, section);

	self:UpdateSlider();
	self:Update();
end

--------------------------------------------------------------------------------
-- UIDropDown initialization function.                                        --
--------------------------------------------------------------------------------
function TradeLinks:SectionDropDownInit()
	for _, skillId in next, lib:GetSkillIds() do
		local info = UIDropDownMenu_CreateInfo();
		info.text = lib:GetSkillName(skillId) or ("(missing data for " .. skillId .. " for build " .. select(2,GetBuildInfo()) .. ")");
		info.func = self.SectionSelect;
		info.arg1 = self;
		info.arg2 = skillId;
		info.value = skillId;
		UIDropDownMenu_AddButton(info);
	end
end
	
--------------------------------------------------------------------------------
-- Update function to refresh the lines.                                      --
--------------------------------------------------------------------------------
function TradeLinks:Update()
	local offset = floor(self.Slider:GetValue()) - 1;	
	local data = self:GetData(self.Section);
	local saved = self:GetSaved(self.Section);
		
	for i = 1, LINE_COUNT do
		local line = self.Lines[i];
		local index = offset + i;
		if data[index] then
			local spellId = data[index];
			line:Show();
			line.Index = index;
			line.SpellId = spellId;
			line.Text:SetText(GetSpellInfo(spellId));
			line:SetChecked(saved[spellId]);
		else
			line:Hide();
		end
	end
end

--------------------------------------------------------------------------------
-- GUI function to create a texture.                                          --
--------------------------------------------------------------------------------
function TradeLinks:CreateTexture(parent, name, layer, texture, width, height, anchor, offsetx, offsety, anchor2)
	local tex = parent:CreateTexture(name, layer);
	tex:SetTexture(texture);
	if width and height then
		tex:SetWidth(width);
		tex:SetHeight(height);
	end
	if anchor then
		tex:SetPoint(anchor, parent, anchor2 or anchor, offsetx, offsety);
	end
	
	return tex;
end

--------------------------------------------------------------------------------
-- GUI function to create the lines and scrollbar.                            --
--------------------------------------------------------------------------------
function TradeLinks:CreateScroll()
	self.Lines = {};
	for i = 1, LINE_COUNT do
		local frame = CreateFrame("CheckButton", nil, self.Frame);		
		frame:SetHeight(22);
		frame:SetWidth(22);
		if i == 1 then
			frame:SetPoint("TOPLEFT", self.Frame, "TOPLEFT", 25, -100);
		else
			frame:SetPoint("TOP", self.Lines[i-1], "BOTTOM", 0, 8);
		end
		
		frame:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up");
		frame:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down");
		frame:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight");
		frame:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check");
		frame:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled");
		frame:SetHitRectInsets(0, -255, 3, 3);

		frame.Text = frame:CreateFontString(nil, "OVERLAY");
		frame.Text:SetFontObject(GameFontNormal);
		frame.Text:SetHeight(14);
		frame.Text:SetWidth(255);
		frame.Text:SetPoint("LEFT", frame, "RIGHT");
		frame.Text:SetJustifyH("LEFT");
		frame.Text:SetText("");
		
		frame:SetScript("OnEnter", function(...) self:OnEnter(...) end);
		frame:SetScript("OnLeave", function(...) self:OnLeave(...) end);
		frame:SetScript("OnClick", function(...) self:OnClick(...) end);
		self.Lines[i] = frame;
	end
	
	local frame = CreateFrame("Slider", nil, self.Frame);
	frame:SetOrientation("VERTICAL");
	frame:SetWidth(16);
	frame:SetPoint("TOPRIGHT", self.Frame, "TOPRIGHT", -46, -94);
	frame:SetPoint("BOTTOMRIGHT", self.Frame, "BOTTOMRIGHT", -46, 77);
	
	local thumb = frame:CreateTexture(nil, "ARTWORK");
	thumb:SetTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
	thumb:SetWidth(16);
	thumb:SetHeight(24);
	thumb:SetTexCoord(0.25, 0.75, 0.125, 0.875);
	
	frame:SetThumbTexture(thumb);
	
	local topTex = frame:CreateTexture(nil, "ARTWORK")
	topTex:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar");
	topTex:SetWidth(31);
	topTex:SetHeight(256);
	topTex:SetTexCoord(0, 124/256, 21/256, 1);
	topTex:SetPoint("TOPLEFT", frame, "TOPRIGHT", -23, -2);
	local bottomTex = frame:CreateTexture(nil, "ARTWORK")
	bottomTex:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar");
	bottomTex:SetWidth(31);
	bottomTex:SetHeight(106);
	bottomTex:SetTexCoord(132/256, 1, 0, 87/256);
	bottomTex:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", -23, 2);
	
	frame.Top = topTex;
	frame.Bottom = bottomTex;
	
	frame:SetScript("OnValueChanged", function() self:Update() end);
	
	self.Slider = frame;
end

--------------------------------------------------------------------------------
-- GUI function to create the main frame, dropdown and buttons.               --
--------------------------------------------------------------------------------
function TradeLinks:CreateFrame()
	local frame = CreateFrame("Frame", "TradeLinks_Frame", UIParent);
	frame:SetFrameStrata("DIALOG");
	frame:SetWidth(384);
	frame:SetHeight(512);
	frame:SetHitRectInsets(0, 30, 0, 45);
	frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, -104);
	frame:EnableMouse(true);
	frame:EnableMouseWheel(true);
	
	frame:RegisterEvent("ADDON_LOADED");
	frame:SetScript("OnEvent", function(this, event, arg1) 
		UIDropDownMenu_Initialize(self.DropDown, function(...) self:SectionDropDownInit(...) end);	
		self:SetSection(lib.SKILL_ALCHEMY);
		this:UnregisterAllEvents();
		this:SetScript("OnEvent", nil);
	end);

	frame:SetScript("OnMouseWheel", function (this, direction)
		local value = self.Slider:GetValue() - 4 * direction;
		self.Slider:SetValue(value);
	end);
	frame:Hide();
	
	self.Frame = frame;
	
	self:CreateTexture(frame, nil, "BACKGROUND", "Interface\\Spellbook\\Spellbook-Icon", 60, 60, "TOPLEFT", 7, -6);	
	self:CreateTexture(frame, nil, "ARTWORK", "Interface\\ClassTrainerFrame\\UI-ClassTrainer-TopLeft", 256, 256, "TOPLEFT", 0, 0);
	self:CreateTexture(frame, nil, "ARTWORK", "Interface\\ClassTrainerFrame\\UI-ClassTrainer-TopRight", 128, 256, "TOPRIGHT", 0, 0);
	self:CreateTexture(frame, nil, "ARTWORK", "Interface\\PaperDollInfoFrame\\UI-Character-General-BottomLeft", 256, 256, "BOTTOMLEFT", 0, 0);
	self:CreateTexture(frame, nil, "ARTWORK", "Interface\\PaperDollInfoFrame\\UI-Character-General-BottomRight", 128, 256, "BOTTOMRIGHT", 0, 0);
	
	local font = frame:CreateFontString(nil, "OVERLAY");
	font:SetFontObject(GameFontNormal);
	font:SetPoint("TOP", 0, -18);
	font:SetText(self.Version);
	
	local button = CreateFrame("Button", nil, frame, "UIPanelCloseButton");
	button:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -30, -8);
	UIPanelWindows["TradeLinks_Frame"] = { area = "left", pushable = 13, whileDead = 1 };
	
	local dropdown = CreateFrame("Frame", "TradeLinks_SectionDropDown", self.Frame, "UIDropDownMenuTemplate");
	dropdown:SetPoint("TOPLEFT", self.Frame, "TOPLEFT", 57, -37);
	self.DropDown = dropdown;
	UIDropDownMenu_SetWidth(self.DropDown, 250);
	
	local click = CreateFrame("Button", "TradeLinksEncode", self.Frame, "GameMenuButtonTemplate");
	click:SetText("Link");
	click:SetWidth(100);
	click:SetHeight(22);
	click:SetPoint("TOPRIGHT", self.Frame, "TOPRIGHT", -42, -70);

	click:SetScript("OnClick", function(frame)
		local link, encoded = lib:Encode(self.Section, self:GetSaved(self.Section));
		if ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Insert(link);
		else
			ChatFrame1:AddMessage(link);
			SetItemRef(encoded);
		end
	end);
	
	local click = CreateFrame("Button", "TradeLinksSelectAll", self.Frame, "GameMenuButtonTemplate");
	click:SetText("Select all");
	click:SetWidth(100);
	click:SetHeight(22);
	click:SetPoint("TOPLEFT", self.Frame, "TOPLEFT", 135, -70);
	
	click:SetScript("OnClick", function(frame)
		local data = self:GetData(self.Section);
		local saved = self:GetSaved(self.Section);
		
		for k, v in next, data do
			saved[v] = true;
		end
		
		self:Update();
	end);
	
	
	local click = CreateFrame("Button", "TradeLinksClear", self.Frame, "GameMenuButtonTemplate");
	click:SetText("Clear");
	click:SetWidth(100);
	click:SetHeight(22);
	click:SetPoint("TOPLEFT", self.Frame, "TOPLEFT", 25, -70);

	click:SetScript("OnClick", function(frame)
		local saved = self:GetSaved(self.Section);
		for k, v in next, saved do
			saved[k] = nil;
		end
		self:Update();
	end);
	
end

--------------------------------------------------------------------------------
-- Init function to create the GUI and set up slash commands.                 --
--------------------------------------------------------------------------------
function TradeLinks:Init() 
	self:CreateFrame();
	self:CreateScroll();
	
	_G.SLASH_TradeLinks1 = "/tradelinks";
	_G.SLASH_TradeLinks2 = "/tl";
	SlashCmdList["TradeLinks"] = function(msg)
		ShowUIPanel(self.Frame);
	end
	
	_G.SLASH_TradeLinksDecode1 = "/tradelinksdecode";
	_G.SLASH_TradeLinksDecode2 = "/tldecode";
	SlashCmdList["TradeLinksDecode"] = function(msg)
		lib:DecodeOut(msg, msg:trim():match("^x"));
	end	
	

	_G.SLASH_TradeLinksMissing1 = "/tlmissing";
	SlashCmdList["TradeLinksMissing"] = function(msg)
		local data, skillId = lib:Decode(msg);
		if not data then
			ChatFrame1:AddMessage("Usage: /tlmissing [trade link]");
		end
		local allData = lib:GetData(skillId);
		local missing = {};
		for k, v in next, allData do
			if not data[v] then
				missing[v] = 1;
			end
		end
		local link, trade = lib:Encode(skillId, missing);
		SetItemRef(trade);
		ChatFrame1:AddMessage("Missing: " .. link);
	end	
end

TradeLinks:Init();


----------------------------
local function TL_comp()
	local data = {};
	for i=1, GetNumTradeSkills() do
		local link = GetTradeSkillRecipeLink(i);
		if link then
			local spellId = tonumber(link:match("enchant:(%d+)"));
			data[spellId] = 1;
		end
	end
	
	return data;
end

function TradeLinks:Test(link)	
	local data1 = lib:Decode(link);	
	local data2 = TL_comp();
	local c1 = 0; for k,v in next,data1 do c1 = c1 + 1; end
	local c2 = 0; for k,v in next,data2 do c2 = c2 + 1; end
	
	ChatFrame1:AddMessage("Decoded to: " .. c1 .. " elements, counted to: " .. c2);
	
	for k,v in next, data1 do
		if data2[k] then
			data2[k] = nil;
		else
			ChatFrame1:AddMessage((GetSpellLink(k) or k) .. " was not found in compare set");
		end
	end
	
	for k,v in next, data2 do
		ChatFrame1:AddMessage((GetSpellLink(k) or k) .. " was not found in original set");
	end
end
